require 'redis'

require 'rubbis/server'

module AcceptanceHelpers
	def client
		Redis.new(host: 'localhost', port: TEST_PORT)
	end

	def with_server(opts = {})
		opts[:server_file] = opts.fetch(:server_file, true)
		server = nil
		server_thread = Thread.new do
			begin
				server = Rubbis::Server.new(
					port: TEST_PORT,
					server_file: opts[:server_file] ? test_db : nil,
					aof_file: opts[:aof_file] ? aof_test_db : nil
					)
				server.listen
			rescue
				puts "[ERRS]"
				puts $!
				puts $!.backtrace
			end
		end

		wait_for_open_port TEST_PORT

		yield
	rescue TimeoutError, Redis::CannotConnectError
		server_thread.value unless server_thread.alive?

		raise
	ensure
		server.shutdown if server
	end

	def wait_for_open_port(port)
		time = Time.now
		while !check_port(port) && time > Time.now - 3
			sleep 0.01
		end
		raise TimeoutError unless check_port(port)
	end

	def check_port(port)
		`nc -z localhost #{port}`
		res = $?.success?
		puts $?.inspect unless res
		res
	end	


end

class FakeClock
	def initialize
		@t = 0
	end

	def now
		@t
	end

	def sleep(seconds)
		@t += seconds
	end
end

TEST_PORT = 6380

RSpec.configure do |c|
	def test_db
		Dir.tmpdir + '/rubbis_test.dump'
	end

	def aof_test_db
		Dir.tmpdir + '/rubbis_test.aof'
	end

	c.before(:each) do
		FileUtils.rm_f(test_db)
	end


	c.include AcceptanceHelpers, acceptance: true
end
